pro read_solar_hdf, filename, status, solar

;  Purpose:
;  This provides the data structures required to read in the solar SAGE III
;  event species data HDF files
;
;  Author:
;  Lawrence Selvy, SSAI
;  Lawrence.D.Selvy@nasa.gov
;
;  Input Variables:
;  filename - Name of the HDF file to be read in (g3a.ssp.*)
;
;  Output Variables;
;  status - Status variable of the read
;  

;  Initialize local varibales

status = 'OK'

save_solar_reader_version = 0

;  Create the structure to read from the HDF file needed to dynamically size
;  the data arrays

solar_size = {event_id:0L, $
                     yr_day_tag:0L, $
                     mission_time:0., $
                     int_fill_value:0L, $
                     flt_fill_value:0., $
                     mission_id:0L, $
                     L0DO_Version:0., $
                     L0_Version:0., $
                     Software_Version:0., $
                     Dataproduct_Version:0.,$
                     DB_Version:0.,$
                     GRAM95_Version:0., $
                     Met_Version:0., $
                     bin_height:0., $
                     num_bins:0L, $
                     num_ao3_channels:0L, $
                     num_grnd_trk:0L, $
                     num_aer_bins:0L}

!QUIET = 1

;  Start to extract the needed information

fileID = HDF_OPEN(filename,/read)

;  Get to the first vdata
vdID = -1
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle, x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parse the information fromt he byte array returned from the read function into 
;  understandable data
solar_size.event_id = long(x,0)
solar_size.yr_day_tag = long(x,4)
solar_size.mission_time = float(x,8)
solar_size.int_fill_value = long(x,12)
solar_size.flt_fill_value = float(x,16)
solar_size.mission_id = long(x,20)
;  Detach this vdata
HDF_VD_DETACH,vdHandle

;  Get the next vdata
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle, x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parse the information from this byte array into understandable data
solar_size.L0DO_Version = float(x,0)
solar_size.L0_Version = float(x,4)
solar_size.DB_Version = float(x,8)
solar_size.GRAM95_Version = float(x,12)
solar_size.Met_Version = float(x,16)
solar_size.Software_Version = float(x,20)
solar_size.Dataproduct_Version = float(x,24)
;  Detach this Vdata
HDF_VD_DETACH,vdHandle

;  Get the next vdata
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle, x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parse the information fromt he Byte array to understandable data
solar_size.bin_height = float(x,0)
solar_size.num_bins = long(x,4)
solar_size.num_ao3_channels = long(x,8)
solar_size.num_grnd_trk = long(x,12)
solar_size.num_aer_bins = long(x,16)
;  Detach the vdata
HDF_VD_DETACH,vdHandle

;  Set version variable
vers = solar_size.Software_Version

;  Create the aerosol extinction channel data
;  NOTE: The tag AerosolExt is an array containing this structure
;        for each channel.  To reference the i-th element of the 
;        AerosolExt, the correct syntax is solar.AerosolExt(i).aerext

aerosol_ext = { aerosol_ext, $
                aerext:fltarr(solar_size.num_aer_bins), $
                aerext_uncert:fltarr(solar_size.num_aer_bins), $
                aerQA:lonarr(solar_size.num_aer_bins)   }

;  Create the data structure to be read in
;  Define the solar structure.
solar = { event_id:0L, $
      yr_day_tag:0L, $
      mission_time:0., $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      L0_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      DB_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      num_bins:0L, $
      num_ao3_channels:0L,$
      num_grnd_trk:0L, $
      num_aer_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle:0., $
      EvtStatusFlag:0L, $
      st_date:0L, $
      st_time:0L, $
      st_latitude:0., $
      st_longitude:0., $
      st_altitude:0., $
      end_date:0L, $
      end_time:0L, $
      end_latitude:0., $
      end_longitude:0., $
      end_altitude:0., $
      gt_date:lonarr(solar_size.num_grnd_trk), $
      gt_time:lonarr(solar_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_size.num_grnd_trk), $
      Homogeneity:lonarr(solar_size.num_bins), $
      geom_alt:fltarr(solar_size.num_bins), $
      potential_alt:fltarr(solar_size.num_bins), $
      CurrentTemp:fltarr(solar_size.num_bins), $
      CurrentTemp_uncert:fltarr(solar_size.num_bins), $
      CurrentPress:fltarr(solar_size.num_bins), $
      CurrentPress_uncert:fltarr(solar_size.num_bins), $
      TPSource:lonarr(solar_size.num_bins), $
      trop_temp:0., $
      trop_alt:0., $
      CompositeO3:fltarr(solar_size.num_bins), $
      CompositeO3_uncert:fltarr(solar_size.num_bins), $
      SPOzone:fltarr(solar_size.num_bins), $
      SPOzone_uncert:fltarr(solar_size.num_bins), $
      CompO3QA:lonarr(solar_size.num_bins), $
      MesO3:fltarr(solar_size.num_bins), $
      MesO3_uncert:fltarr(solar_size.num_bins), $
      MesSPOzone:fltarr(solar_size.num_bins), $
      MesSPOzone_uncert:fltarr(solar_size.num_bins), $
      MesO3QA:lonarr(solar_size.num_bins), $
      ChapO3:fltarr(solar_size.num_bins), $
      ChapO3_uncert:fltarr(solar_size.num_bins), $
      ChapSPOzone:fltarr(solar_size.num_bins), $
      ChapSPOzone_uncert:fltarr(solar_size.num_bins), $
      MLRO3QA:lonarr(solar_size.num_bins), $
      Ozone_aO3:fltarr(solar_size.num_bins), $
      Ozone_aO3_uncert:fltarr(solar_size.num_bins), $
      aO3SPOzone:fltarr(solar_size.num_bins), $
      aO3SPOzone_uncert:fltarr(solar_size.num_bins), $
      Ozone_aO3QA:lonarr(solar_size.num_bins), $
      WaterVaporNDP:fltarr(solar_size.num_bins), $
      WaterVaporNDP_uncert:fltarr(solar_size.num_bins), $
      H2OQA:lonarr(solar_size.num_bins), $
      NO2NDP:fltarr(solar_size.num_bins), $
      NO2NDP_uncert:fltarr(solar_size.num_bins), $
      SPNO2:fltarr(solar_size.num_bins), $
      SPNO2_uncert:fltarr(solar_size.num_bins), $
      NO2QA:lonarr(solar_size.num_bins), $
      RetTemp:fltarr(solar_size.num_bins), $
      RetTemp_uncert:fltarr(solar_size.num_bins), $
      RetPress:fltarr(solar_size.num_bins), $
      RetPress_uncert:fltarr(solar_size.num_bins), $
      RetTPQA:lonarr(solar_size.num_bins), $
      Aer_wavelength:fltarr(solar_size.num_ao3_channels), $
      Aer_width:fltarr(solar_size.num_ao3_channels), $
      StratOD:fltarr(solar_size.num_ao3_channels), $
      StratOD_uncert:fltarr(solar_size.num_ao3_channels), $
      StratODQA:lonarr(solar_size.num_ao3_channels), $
      AerosolExt:replicate(aerosol_ext, solar_size.num_ao3_channels) , $
      asd_flag:lonarr(solar_size.num_aer_bins), $
      Aerosol1020:fltarr(solar_size.num_aer_bins), $
      Aerosol1020_uncert:fltarr(solar_size.num_aer_bins), $
      Aero1020QA:lonarr(solar_size.num_aer_bins) } 

;  Now we read in the data from the solar_size array
solar.event_id = solar_size.event_id
solar.yr_day_tag = solar_size.yr_day_tag
solar.mission_time = solar_size.mission_time
solar.int_fill_value = solar_size.int_fill_value
solar.flt_fill_value = solar_size.flt_fill_value
solar.mission_id = solar_size.mission_id
solar.L0DO_Version = solar_size.L0DO_Version
solar.L0_Version = solar_size.L0_Version
solar.Software_Version = solar_size.Software_Version
solar.Dataproduct_Version = solar_size.Dataproduct_Version
solar.DB_Version = solar_size.DB_Version
solar.GRAM95_Version = solar_size.GRAM95_Version
solar.Met_Version = solar_size.Met_Version
solar.bin_height = solar_size.bin_height
solar.num_bins = solar_size.num_bins
solar.num_ao3_channels = solar_size.num_ao3_channels
solar.num_grnd_trk = solar_size.num_grnd_trk
solar.num_aer_bins = solar_size.num_aer_bins

; Now we read in the rest of the data from the HDF file
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
solar.sc_evt_type = long(x,0)
solar.gnd_evt_type = long(x,4)
solar.BetaAngle = float(x,8)
solar.EvtStatusFlag = long(x,12)
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
solar.st_date = long(x,0)
solar.st_time = long(x,4)
solar.st_latitude = float(x,8)
solar.st_longitude = float(x,12)
solar.st_altitude = float(x,16)
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
solar.end_date = long(x,0)
solar.end_time = long(x,4)
solar.end_latitude = float(x,8)
solar.end_longitude = float(x,12)
solar.end_altitude = float(x,16)
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_grnd_trk then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L, solar_size.num_grnd_trk-1 DO BEGIN
   solar.gt_date(k) = long(x,index)
   index = index + 4
   solar.gt_time(k) = long(x,index)
   index = index + 4
   solar.gt_latitude(k) = float(x,index)
   index = index + 4
   solar.gt_longitude(k) = float(x,index)
   index = index + 4
   solar.gt_ray_dir(k) = float(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_bins then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L, solar_size.num_bins-1 DO BEGIN
   solar.Homogeneity(k) = long(x,index)
   index = index + 4
   solar.geom_alt(k) = float(x,index)
   index = index + 4
   solar.potential_alt(k) = float(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_bins then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L,solar_size.num_bins-1 DO BEGIN
   solar.CurrentTemp(k) = float(x,index)
   index = index + 4
   solar.CurrentTemp_uncert(k) = float(x,index)
   index = index + 4
   solar.CurrentPress(k) = float(x,index)
   index = index + 4
   solar.CurrentPress_uncert(k) = float(x,index)
   index = index + 4
   solar.TPSource(k) = long(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
solar.trop_temp = float(x,0)
solar.trop_alt = float(x,4)
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_bins then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L,solar_size.num_bins-1 DO BEGIN
   solar.CompositeO3(k) = float(x,index)
   index = index + 4
   solar.CompositeO3_uncert(k) = float(x,index)
   index = index + 4
   solar.SPOzone(k) = float(x,index)
   index = index + 4
   solar.SPOzone_uncert(k) = float(x,index)
   index = index + 4
   solar.CompO3QA(k) = long(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_bins then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L,solar_size.num_bins-1 DO BEGIN
   solar.MesO3(k) = float(x,index)
   index = index + 4
   solar.MesO3_uncert(k) = float(x,index)
   index = index + 4
   solar.MesSPOzone(k) = float(x,index)
   index = index + 4
   solar.MesSPOzone_uncert(k) = float(x,index)
   index = index + 4
   solar.MesO3QA(k) = long(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_bins then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L, solar_size.num_bins-1 DO BEGIN
   solar.ChapO3(k) = float(x,index)
   index = index + 4
   solar.ChapO3_uncert(k) = float(x,index)
   index = index + 4
   solar.ChapSPOzone(k) = float(x,index)
   index = index + 4
   solar.ChapSPOzone_uncert(k) = float(x,index)
   index = index + 4
   solar.MLRO3QA(k) = long(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_bins then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L, solar_size.num_bins-1 DO BEGIN
   solar.Ozone_ao3(k) = float(x,index)
   index = index + 4
   solar.Ozone_ao3_uncert(k) = float(x,index)
   index = index + 4
   solar.aO3SPOzone(k) = float(x,index)
   index = index + 4
   solar.aO3SPOzone_uncert(k) = float(x,index)
   index = index + 4
   solar.Ozone_aO3QA(k) = long(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_bins then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L, solar_size.num_bins-1 DO BEGIN
   solar.WaterVaporNDP(k) = float(x,index)
   index = index + 4
   solar.WaterVaporNDP_uncert(k) = float(x,index)
   index = index + 4
   solar.H2OQA(k) = long(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_bins then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L, solar_size.num_bins-1 DO BEGIN
   solar.NO2NDP(k) = float(x,index)
   index = index + 4
   solar.NO2NDP_uncert(k) = float(x,index)
   index = index + 4
   solar.SPNO2(k) = float(x,index)
   index = index + 4
   solar.SPNO2_uncert(k) = float(x,index)
   index = index + 4
   solar.NO2QA(k) = long(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_bins then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L, solar_size.num_bins-1 DO BEGIN
   solar.RetTemp(k) = float(x,index)
   index = index + 4
   solar.RetTemp_uncert(k) = float(x,index)
   index = index + 4
   solar.RetPress(k) = float(x,index)
   index = index + 4
   solar.RetPress_uncert(k) = float(x,index)
   index = index + 4
   solar.RetTPQA(k) = float(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_ao3_channels then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L, solar_size.num_ao3_channels-1 DO BEGIN
   solar.Aer_wavelength(k) = float(x,index)
   index = index + 4
   solar.Aer_width(k) = float(x,index)
   index = index + 4
   solar.StratOD(k) = float(x,index)
   index = index + 4
   solar.StratOD_uncert(k) = float(x,index)
   index = index + 4
   solar.StratODQA(k) = long(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_aer_bins then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L, solar_size.num_aer_bins-1 DO BEGIN
   for i = 0L, solar_size.num_ao3_channels-1 DO BEGIN
      solar.AerosolExt(i).aerext(k) = float(x,index)
      index = index + 4
      solar.AerosolExt(i).aerext_uncert(k) = float(x,index)
      index = index + 4
      solar.AerosolExt(i).aerQA(k) = long(x,index)
      index = index + 4
   endfor
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_size.num_aer_bins then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L, solar_size.num_aer_bins-1 DO BEGIN
   solar.Aerosol1020(k) = float(x,index)
   index = index + 4
   solar.Aerosol1020_uncert(k) = float(x,index)
   index = index + 4
   solar.Aero1020QA(k) = float(x,index)
   index = index + 4
   solar.asd_flag(k) = long(x,index)
   index = index + 4
endfor
;Close this vdata
HDF_VD_DETACH,vdHandle

;  Now that all the data is read in, we now close the file 
HDF_CLOSE,fileID

end
