pro read_solar_trans_hdf, filename, status, solar_trans

;  Purpose:
;  This provides the data structures required to read in the SAGE III
;  Solar Transmission HDF4 product files.
;
;  Author:
;  Marsha LaRosee, SSAI, marsha.a.larosee@nasa.gov
;
;  Input Variables:
;  filename - Name of the HDF file to be read in (g3b.t.*)
;
;  Output Variables;
;  status - Status variable of the read
;  

;  Initialize local varibales

status = 'OK'

;  Create the structure to read from the HDF file needed to dynamically size
;  the data arrays

solar_trans_size = {solar_trans_size, event_id:0L, $
                 yr_day_tag:0L, $
                 mission_time:0., $
                 int_fill_value:0L, $
                 flt_fill_value:0., $
                 mission_id:0L, $
                 L0DO_Version:0., $
                 L0_Version:0., $
                 Software_Version:0., $
                 Dataproduct_Version:0., $
                 DB_Version:0., $
                 GRAM95_Version:0., $
                 Met_Version:0., $
                 bin_height:0., $
                 profile_count:0L, $
                 num_grnd_trk:0L, $
                 num_press_grid:0L, $
                 num_CCDPxlGrps:0L, $
                 num_alt_bins:0L }

!QUIET = 1

;  Start to extract the needed information

fileID = HDF_OPEN(filename,/read)

;  Get to the first vdata
vdID = -1
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle, x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parse the information from the byte array returned from the read function into 
;  understandable data
solar_trans_size.event_id = long(x,0)
solar_trans_size.yr_day_tag = long(x,4)
solar_trans_size.mission_time = float(x,8)
solar_trans_size.int_fill_value = long(x,12)
solar_trans_size.flt_fill_value = float(x,16)
solar_trans_size.mission_id = long(x,20)
;  Detach this vdata
HDF_VD_DETACH,vdHandle

;  Get the next vdata
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle, x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parse the information from this byte array into understandable data
solar_trans_size.L0DO_Version = float(x,0)
solar_trans_size.L0_Version = float(x,4)
solar_trans_size.DB_Version = float(x,8)
solar_trans_size.GRAM95_Version = float(x,12)
solar_trans_size.Met_Version = float(x,16)
solar_trans_size.Software_Version = float(x,20)
solar_trans_size.Dataproduct_Version = float(x,24)
;  Detach this Vdata
HDF_VD_DETACH,vdHandle

;  Get the next vdata
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle, x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parse the information from the Byte array to understandable data
solar_trans_size.bin_height = float(x,0)
solar_trans_size.profile_count = long(x,4)
solar_trans_size.num_grnd_trk = long(x,8)
solar_trans_size.num_press_grid = long(x,12)
solar_trans_size.num_CCDPxlGrps = long(x,16)
solar_trans_size.num_alt_bins = long(x,20)
;  Detach the vdata
HDF_VD_DETACH,vdHandle

;  Set version variable
vers = solar_trans_size.Software_Version

;  NOTE:  The tag pg_trans_profiles is an array containing this structure
;         for each channel.  To reference the i-th element of the
;         array, an example of the correct syntax is 
;         solar_trans.pg_trans_profiles(i).Transmission

pg_trans_profile = { pg_trans_profile, $
                      GeometricAltitude:fltarr(solar_trans_size.num_alt_bins), $
                      Transmission:fltarr(solar_trans_size.num_alt_bins), $
                      Transmission_unc:fltarr(solar_trans_size.num_alt_bins), $
                      TransQA:lonarr(solar_trans_size.num_alt_bins) }


;  Create the data structure to be read in
;  Define the solar structure.
solar_trans = { event_id:0L, $
      yr_day_tag:0L, $
      mission_time:0., $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      L0_Version:0., $
      DB_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      bin_height:0., $
      profile_count:0L,$
      num_grnd_trk:0L, $
      num_press_grid:0L, $
      num_CCDPxlGrps:0L, $
      num_alt_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle:0., $
      EvtStatusFlag:0L, $
      st_date:0L, $
      st_time:0L, $
      st_latitude:0., $
      st_longitude:0., $
      st_altitude:0., $
      end_date:0L, $
      end_time:0L, $
      end_latitude:0., $
      end_longitude:0., $
      end_altitude:0., $
      gt_date:lonarr(solar_trans_size.num_grnd_trk), $
      gt_time:lonarr(solar_trans_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_trans_size.num_grnd_trk), $
      trop_temp:0., $
      trop_alt:0., $
      met_pressure:fltarr(solar_trans_size.num_press_grid), $
      met_temp:fltarr(solar_trans_size.num_press_grid), $
      met_temp_unc:fltarr(solar_trans_size.num_press_grid), $
      met_altitude:fltarr(solar_trans_size.num_press_grid), $
      met_Source:0L, $
      start_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      end_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      central_wavelength:fltarr(solar_trans_size.num_CCDPxlGrps), $
      half_bandwidth:fltarr(solar_trans_size.num_CCDPxlGrps), $
      Altitude:fltarr(solar_trans_size.num_alt_bins), $
      CurrentPress:fltarr(solar_trans_size.num_alt_bins), $
      CurrentTemp:fltarr(solar_trans_size.num_alt_bins), $
      CurrentTemp_uncert:fltarr(solar_trans_size.num_alt_bins), $
      TPSource:lonarr(solar_trans_size.num_alt_bins), $ 
      pg_trans_profiles:replicate( pg_trans_profile , 87 ) }

;  Now we read in the data from the solar_size array
solar_trans.event_id = solar_trans_size.event_id
solar_trans.yr_day_tag = solar_trans_size.yr_day_tag
solar_trans.mission_time = solar_trans_size.mission_time
solar_trans.int_fill_value = solar_trans_size.int_fill_value
solar_trans.flt_fill_value = solar_trans_size.flt_fill_value
solar_trans.mission_id = solar_trans_size.mission_id
solar_trans.L0DO_Version = solar_trans_size.L0DO_Version
solar_trans.L0_Version = solar_trans_size.L0_Version
solar_trans.Software_Version = solar_trans_size.Software_Version
solar_trans.Dataproduct_Version = solar_trans_size.Dataproduct_Version
solar_trans.DB_Version = solar_trans_size.DB_Version
solar_trans.GRAM95_Version = solar_trans_size.GRAM95_Version
solar_trans.Met_Version = solar_trans_size.Met_Version
solar_trans.bin_height = solar_trans_size.bin_height
solar_trans.profile_count = solar_trans_size.profile_count
solar_trans.num_grnd_trk = solar_trans_size.num_grnd_trk
solar_trans.num_press_grid = solar_trans_size.num_press_grid
solar_trans.num_CCDPxlGrps = solar_trans_size.num_CCDPxlGrps
solar_trans.num_alt_bins = solar_trans_size.num_alt_bins

; Now we read in the rest of the data from the HDF file
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
solar_trans.sc_evt_type = long(x,0)
solar_trans.gnd_evt_type = long(x,4)
solar_trans.BetaAngle = float(x,8)
solar_trans.EvtStatusFlag = long(x,12)
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
solar_trans.st_date = long(x,0)
solar_trans.st_time = long(x,4)
solar_trans.st_latitude = float(x,8)
solar_trans.st_longitude = float(x,12)
solar_trans.st_altitude = float(x,16)
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
solar_trans.end_date = long(x,0)
solar_trans.end_time = long(x,4)
solar_trans.end_latitude = float(x,8)
solar_trans.end_longitude = float(x,12)
solar_trans.end_altitude = float(x,16)
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_trans_size.num_grnd_trk then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L, solar_trans_size.num_grnd_trk-1 DO BEGIN
   solar_trans.gt_date(k) = long(x,index)
   index = index + 4
   solar_trans.gt_time(k) = long(x,index)
   index = index + 4
   solar_trans.gt_latitude(k) = float(x,index)
   index = index + 4
   solar_trans.gt_longitude(k) = float(x,index)
   index = index + 4
   solar_trans.gt_ray_dir(k) = float(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
solar_trans.trop_temp = float(x,0)
solar_trans.trop_alt = float(x,4)
solar_trans.met_source = long(x,8)
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_trans_size.num_press_grid then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L, solar_trans_size.num_press_grid-1 DO BEGIN
   solar_trans.met_pressure(k) = float(x,index)
   index = index + 4
   solar_trans.met_temp(k) = float(x,index)
   index = index + 4
   solar_trans.met_temp_unc(k) = float(x,index)
   index = index + 4
   solar_trans.met_altitude(k) = float(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_trans_size.num_CCDPxlGrps then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L,solar_trans_size.num_CCDPxlGrps-1 DO BEGIN
   solar_trans.start_pixel_num(k) = long(x,index)
   index = index + 4
   solar_trans.end_pixel_num(k) = long(x,index)
   index = index + 4
   solar_trans.central_wavelength(k) = float(x,index)
   index = index + 4
   solar_trans.half_bandwidth(k) = float(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne solar_trans_size.num_alt_bins then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parsing the data from the byte array into understandable data
index = 0
for k = 0L,solar_trans_size.num_alt_bins-1 DO BEGIN
   solar_trans.Altitude(k) = float(x,index)
   index = index + 4
   solar_trans.CurrentPress(k) = float(x,index)
   index = index + 4
   solar_trans.CurrentTemp(k) = float(x,index)
   index = index + 4
   solar_trans.CurrentTemp_uncert(k) = float(x,index)
   index = index + 4
   solar_trans.TPSource(k) = long(x,index)
   index = index + 4
endfor
;  Close this vdata
HDF_VD_DETACH,vdHandle

for i = 0L,solar_trans_size.num_CCDPxlGrps DO BEGIN
   vdID = HDF_VD_GETID(fileID,vdID)
   vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
   nrecs = HDF_VD_READ(vdHandle,x)
   if nrecs ne solar_trans_size.num_alt_bins then begin
      print,'There is an error in the HDF file ',vdID
      return
   endif
   ;  Parsing the data from the byte array into understandable data
   index = 0
   for k = 0L,solar_trans_size.num_alt_bins-1 DO BEGIN
      solar_trans.pg_trans_profiles(i).GeometricAltitude(k) = float(x,index)
      index = index + 4
      solar_trans.pg_trans_profiles(i).Transmission(k) = float(x,index)
      index = index + 4
      solar_trans.pg_trans_profiles(i).Transmission_unc(k) = float(x,index)
      index = index + 4
      solar_trans.pg_trans_profiles(i).TransQA(k) = long(x,index)
      index = index + 4
   endfor
   ;  Close this vdata
   HDF_VD_DETACH,vdHandle
endfor
;  Now that all the data is read in, we now close the file 
HDF_CLOSE,fileID

end
